unit VM8095Progr;

interface
uses
  Windows;

procedure RunPIC; stdcall;
procedure Erase; stdcall;
function Complete: integer; stdcall;
procedure PreSend64(address: integer); stdcall;
procedure Read64(address: integer; data: Pointer); stdcall;
procedure Tx64(data: pointer) ; stdcall;
function OpenDevices: integer ; stdcall;
procedure CloseDevices; stdcall;
function SendButtonCommand(command: integer): integer; stdcall;
function VersionFirmware: integer; stdcall;
function IsRunning: integer; stdcall;
function PlayMusic: integer; stdcall;

procedure OutIn(Tx:integer; Rx:integer );

var
  open: boolean;
  send_buf:array[0..63] of byte;
  receive_buf:array[0..63] of byte;
  hWinUsbHandle: THandle;
  hDeviceWrite: THandle;
  hDeviceRead: THandle;

implementation

uses findpath;


function OpenDevices: integer; stdcall;
begin
  CloseDevices;
  givepath;
  if open then
  begin
    result:=1;
  end
  else
    result:=-1;
end;

procedure CloseDevices; stdcall;
begin
  if open then CloseHandle(hDeviceWrite);
  if open then CloseHandle(hDeviceRead);
  open:=false;
end;

procedure PreSend64(address: integer); stdcall;
var tmp: integer;
begin
  send_buf[0] := 2;                     // init Tx64
  tmp:=address shr 8;
  send_buf[1] := tmp;                   // hi byte
  send_buf[2] := address;               // lo byte
  OutIn(3,1);
end;

procedure Tx64(data: Pointer); stdcall;
var p:^byte;
i:integer;
begin
  p:=data;
  for i:=0 to 63 do
  begin
    send_buf[i]:=p^;
    inc(p);
  end;
  OutIn(64,1);
end;


procedure Read64(address: integer; data: Pointer); stdcall;
var p:^byte;
i, tmp:integer;
begin
  send_buf[0] := 3;                     // init Rx64
  tmp:=address shr 8;
  send_buf[1] := tmp;                   // hi byte
  send_buf[2] := address;               // lo byte
  OutIn(3,64);
  p:=data;
  for i:=0 to 63 do
  begin
    p^:=receive_buf[i];
    inc(p);
  end;
end;

procedure Erase; stdcall
begin
    send_buf[0] := 1;  // Erase FLASH
    OutIn(1,0);
end;

function Complete: integer;  stdcall;
begin
    send_buf[0] := 5;  // Check Erase complete
    OutIn(1,1);
    result:=receive_buf[0];
end;

procedure RunPIC;  stdcall
begin
    send_buf[0] := 4;  // Run PIC
    OutIn(1,0);
end;

function PlayMusic: integer; stdcall;
begin
    send_buf[0] := 7;  // Play music
    OutIn(1,1);
    result:=receive_buf[1];
end;

function SendButtonCommand(command: integer): integer; stdcall;
begin
    send_buf[0] := 10;  // Button Command
    send_buf[1] := command;
    OutIn(2,1);
    result:=receive_buf[1];     // 10 if OK
end;

function VersionFirmware: integer; stdcall;
begin
    send_buf[0] := 11;  // READ_VERSION
    OutIn(1,1);
    result:=receive_buf[1]*256*256*256+receive_buf[2]*256*256+receive_buf[3]*256+receive_buf[4];
end;

function IsRunning: integer; stdcall;
begin
    send_buf[0] := 12;  // is running
    OutIn(1,1);
    result:=receive_buf[1];
end;

procedure OutIn(Tx:integer; Rx:integer);
var BytesWritten, BytesRead:DWORD;
i:integer;
Buf_tx:array[0..64] of byte;
Buf_rx:array[0..64] of byte;
begin
  for i:=0 to 63 do
     Buf_tx[i+1]:=send_buf[i];
  Buf_tx[0]:=0;
  BytesRead:=0;
  if open and (Tx>0) then
  begin
    WriteFile(hDeviceWrite,Buf_tx,65,BytesWritten,nil);
    if BytesWritten=0 then
    begin
      open:=false;
      CloseHandle(hDeviceWrite);
      CloseHandle(hDeviceRead);
    end;
  end;
  if open and (Rx>0) then
  begin
    ReadFile(hDeviceRead,Buf_rx,65,BytesRead,nil);
    for i:=0 to 63 do
      receive_buf[i]:=Buf_rx[i+1];
    if BytesRead=0 then
    begin
      open:=false;
      CloseHandle(hDeviceWrite);
      CloseHandle(hDeviceRead);
    end;
  end;
end;

end.
